#include "mainwindow.h"

enum {
	COL_TITLE = 0,
	COL_DATE,
	COL_TAGS,
	COL_TIMESTAMP,
	COL_COUNT
};

static gint
mainWindow_sort_date_compare_func(GtkTreeModel *model,
				  GtkTreeIter  *a,
				  GtkTreeIter  *b,
				  gpointer      userdata)
{
	guint time1, time2;

	gtk_tree_model_get(model, a, COL_TIMESTAMP, &time1, -1);
	gtk_tree_model_get(model, b, COL_TIMESTAMP, &time2, -1);

	if (time1 == time2)
		return 0;
	else if (time1 > time2)
		return 1;
	else
		return -1;
}


static void mainWindow_configure_treeView(struct mainWindow *mainWindow)
{
	GtkCellRenderer *renderer;
	GtkTreeIter iter;
	GtkTreeSortable *sortable;
	GtkTreeViewColumn *col;

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(mainWindow->entryListView, -1, "Title", renderer,
						    "text", COL_TITLE, NULL);
	col = gtk_tree_view_get_column(mainWindow->entryListView, COL_TITLE);
	gtk_tree_view_column_set_resizable(col, 1);
	gtk_tree_view_column_set_sort_column_id(col, COL_TITLE);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(mainWindow->entryListView, -1, "Date", renderer,
						    "text", COL_DATE, NULL);
	col = gtk_tree_view_get_column(mainWindow->entryListView, COL_DATE);
	gtk_tree_view_column_set_resizable(col, 1);
	gtk_tree_view_column_set_sort_column_id(col, COL_DATE);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(mainWindow->entryListView, -1, "Tags", renderer,
						    "text", COL_TAGS, NULL);
	col = gtk_tree_view_get_column(mainWindow->entryListView, COL_TAGS);
	gtk_tree_view_column_set_resizable(col, 1);
	gtk_tree_view_column_set_sort_column_id(col, COL_TAGS);

	sortable = GTK_TREE_SORTABLE(mainWindow->entryListStore);
	gtk_tree_sortable_set_sort_func(sortable, COL_DATE, mainWindow_sort_date_compare_func,
					GINT_TO_POINTER(COL_DATE), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, COL_DATE, GTK_SORT_DESCENDING);

	/*
	 * Code just for debugging - some test values
	 */
	gtk_list_store_append(mainWindow->entryListStore, &iter);
	gtk_list_store_set(mainWindow->entryListStore, &iter,
			   COL_TITLE, "ZA Test entry longer title",
			   COL_DATE, "Weekday, Month nth Year hh:mm",
			   COL_TAGS, "X",
			   COL_TIMESTAMP, 123,
			   -1);
	gtk_list_store_append(mainWindow->entryListStore, &iter);
	gtk_list_store_set(mainWindow->entryListStore, &iter,
			   COL_TITLE, "baz",
			   COL_DATE, "Mayday, July 8th 1234",
			   COL_TAGS, "A",
			   COL_TIMESTAMP, 555,
			   -1);
	gtk_list_store_append(mainWindow->entryListStore, &iter);
	gtk_list_store_set(mainWindow->entryListStore, &iter,
			   COL_TITLE, "Foo",
			   COL_DATE, "Sunday, Movember 1337",
			   COL_TAGS, "F",
			   COL_TIMESTAMP, 11,
			   -1);
}

struct mainWindow *mainWindow_new()
{
	GtkBuilder *builder;
	struct mainWindow *mainWindow;

	builder = gtk_builder_new_from_file("glade/mainWindow.glade");

	mainWindow = malloc(sizeof(struct mainWindow));
	mainWindow->mainWindow            = GTK_WIDGET(gtk_builder_get_object(builder, "mainWindow"));
	mainWindow->buttonChooseWorkspace = GTK_WIDGET(gtk_builder_get_object(builder, "buttonChooseWorkspace"));
	mainWindow->searchEntry           = GTK_WIDGET(gtk_builder_get_object(builder, "searchEntry"));
	mainWindow->dateStart             = GTK_WIDGET(gtk_builder_get_object(builder, "dateStart"));
	mainWindow->dateEnd               = GTK_WIDGET(gtk_builder_get_object(builder, "dateEnd"));
	mainWindow->entryText             = GTK_WIDGET(gtk_builder_get_object(builder, "entryText"));
	mainWindow->entryListView         = GTK_TREE_VIEW(gtk_builder_get_object(builder, "entryListView"));
	mainWindow->entryListStore        = GTK_LIST_STORE(gtk_builder_get_object(builder, "entryListStore"));

	gtk_builder_connect_signals(builder, NULL);

	g_object_unref(G_OBJECT(builder));

	mainWindow_configure_treeView(mainWindow);

	return mainWindow;
}
